@@ -12,23 +12,24 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/docker-slim/docker-slim/pkg/aflag"
-	"github.com/docker-slim/docker-slim/pkg/app"
-	"github.com/docker-slim/docker-slim/pkg/app/master/config"
-	"github.com/docker-slim/docker-slim/pkg/app/master/docker/dockerhost"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/image"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/ipc"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/sensor"
-	"github.com/docker-slim/docker-slim/pkg/app/master/security/apparmor"
-	"github.com/docker-slim/docker-slim/pkg/app/master/security/seccomp"
-	"github.com/docker-slim/docker-slim/pkg/docker/dockerutil"
-	"github.com/docker-slim/docker-slim/pkg/ipc/channel"
-	"github.com/docker-slim/docker-slim/pkg/ipc/command"
-	"github.com/docker-slim/docker-slim/pkg/ipc/event"
-	"github.com/docker-slim/docker-slim/pkg/report"
-	"github.com/docker-slim/docker-slim/pkg/util/errutil"
-	"github.com/docker-slim/docker-slim/pkg/util/fsutil"
-	v "github.com/docker-slim/docker-slim/pkg/version"
+	"github.com/slimtoolkit/slim/pkg/aflag"
+	"github.com/slimtoolkit/slim/pkg/app"
+	"github.com/slimtoolkit/slim/pkg/app/master/config"
+	"github.com/slimtoolkit/slim/pkg/app/master/docker/dockerhost"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/image"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/ipc"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/sensor"
+	"github.com/slimtoolkit/slim/pkg/app/master/security/apparmor"
+	"github.com/slimtoolkit/slim/pkg/app/master/security/seccomp"
+	"github.com/slimtoolkit/slim/pkg/docker/dockerutil"
+	"github.com/slimtoolkit/slim/pkg/ipc/channel"
+	"github.com/slimtoolkit/slim/pkg/ipc/command"
+	"github.com/slimtoolkit/slim/pkg/ipc/event"
+	"github.com/slimtoolkit/slim/pkg/report"
+	"github.com/slimtoolkit/slim/pkg/util/errutil"
+	"github.com/slimtoolkit/slim/pkg/util/fsutil"
+	"github.com/slimtoolkit/slim/pkg/util/jsonutil"
+	v "github.com/slimtoolkit/slim/pkg/version"
 
 	containertypes "github.com/docker/docker/api/types/container"
 	dockerapi "github.com/fsouza/go-dockerclient"
@@ -49,6 +50,7 @@ const (
 	SensorMountPat       = "%s:/opt/_slim/bin/slim-sensor:ro"
 	VolumeSensorMountPat = "%s:/opt/_slim/bin:ro"
 	LabelName            = "_slim"
+	MondelArtifactTar    = "mondel.tar"
 )
 
 type ovars = app.OutVars
@@ -105,13 +107,16 @@ type Inspector struct {
 	DNSServers            []string
 	DNSSearchDomains      []string
 	DoShowContainerLogs   bool
+	DoEnableMondel        bool
 	RunTargetAsUser       bool
 	KeepPerms             bool
 	PathPerms             map[string]*fsutil.AccessInfo
 	ExcludePatterns       map[string]*fsutil.AccessInfo
+	DoExcludeVarLockFiles bool
 	PreservePaths         map[string]*fsutil.AccessInfo
 	IncludePaths          map[string]*fsutil.AccessInfo
 	IncludeBins           map[string]*fsutil.AccessInfo
+	IncludeDirBinsList    map[string]*fsutil.AccessInfo
 	IncludeExes           map[string]*fsutil.AccessInfo
 	DoIncludeShell        bool
 	DoIncludeWorkdir      bool
@@ -121,7 +126,9 @@ type Inspector struct {
 	DoIncludeCertPKAll    bool
 	DoIncludeCertPKDirs   bool
 	DoIncludeNew          bool
+	DoIncludeSSHClient    bool
 	DoIncludeOSLibsNet    bool
+	DoIncludeZoneInfo     bool
 	SelectedNetworks      map[string]NetNameInfo
 	DoDebug               bool
 	LogLevel              string
@@ -181,13 +188,16 @@ func NewInspector(
 	dnsServers []string,
 	dnsSearchDomains []string,
 	showContainerLogs bool,
+	doEnableMondel bool,
 	runTargetAsUser bool,
 	keepPerms bool,
 	pathPerms map[string]*fsutil.AccessInfo,
 	excludePatterns map[string]*fsutil.AccessInfo,
+	doExcludeVarLockFiles bool,
 	preservePaths map[string]*fsutil.AccessInfo,
 	includePaths map[string]*fsutil.AccessInfo,
 	includeBins map[string]*fsutil.AccessInfo,
+	includeDirBinsList map[string]*fsutil.AccessInfo,
 	includeExes map[string]*fsutil.AccessInfo,
 	doIncludeShell bool,
 	doIncludeWorkdir bool,
@@ -197,7 +207,9 @@ func NewInspector(
 	doIncludeCertPKAll bool,
 	doIncludeCertPKDirs bool,
 	doIncludeNew bool,
+	doIncludeSSHClient bool,
 	doIncludeOSLibsNet bool,
+	doIncludeZoneInfo bool,
 	selectedNetworks map[string]NetNameInfo,
 	//serviceAliases []string,
 	doDebug bool,
@@ -234,13 +246,16 @@ func NewInspector(
 		DNSServers:            dnsServers,
 		DNSSearchDomains:      dnsSearchDomains,
 		DoShowContainerLogs:   showContainerLogs,
+		DoEnableMondel:        doEnableMondel,
 		RunTargetAsUser:       runTargetAsUser,
 		KeepPerms:             keepPerms,
 		PathPerms:             pathPerms,
 		ExcludePatterns:       excludePatterns,
+		DoExcludeVarLockFiles: doExcludeVarLockFiles,
 		PreservePaths:         preservePaths,
 		IncludePaths:          includePaths,
 		IncludeBins:           includeBins,
+		IncludeDirBinsList:    includeDirBinsList,
 		IncludeExes:           includeExes,
 		DoIncludeShell:        doIncludeShell,
 		DoIncludeWorkdir:      doIncludeWorkdir,
@@ -250,7 +265,9 @@ func NewInspector(
 		DoIncludeCertPKAll:    doIncludeCertPKAll,
 		DoIncludeCertPKDirs:   doIncludeCertPKDirs,
 		DoIncludeNew:          doIncludeNew,
+		DoIncludeSSHClient:    doIncludeSSHClient,
 		DoIncludeOSLibsNet:    doIncludeOSLibsNet,
+		DoIncludeZoneInfo:     doIncludeZoneInfo,
 		SelectedNetworks:      selectedNetworks,
 		DoDebug:               doDebug,
 		LogLevel:              logLevel,
@@ -268,39 +285,22 @@ func NewInspector(
 	}
 
 	if overrides == nil {
-		inspector.FatContainerCmd = append(inspector.FatContainerCmd, imageInspector.ImageInfo.Config.Entrypoint...)
-		inspector.FatContainerCmd = append(inspector.FatContainerCmd, imageInspector.ImageInfo.Config.Cmd...)
+		inspector.FatContainerCmd = BuildStartupCommand(
+			imageInspector.ImageInfo.Config.Entrypoint,
+			imageInspector.ImageInfo.Config.Cmd,
+			imageInspector.ImageInfo.Config.Shell,
+			false, nil, false, nil,
+		)
 	} else {
-		if len(overrides.Entrypoint) > 0 || overrides.ClearEntrypoint {
-			inspector.FatContainerCmd = append(inspector.FatContainerCmd, overrides.Entrypoint...)
-
-			if len(overrides.Cmd) > 0 || overrides.ClearCmd {
-				inspector.FatContainerCmd = append(inspector.FatContainerCmd, overrides.Cmd...)
-			}
-			//note: not using CMD from image if there's an override for ENTRYPOINT
-		} else {
-			inspector.FatContainerCmd = append(inspector.FatContainerCmd, imageInspector.ImageInfo.Config.Entrypoint...)
-
-			if len(overrides.Cmd) > 0 || overrides.ClearCmd {
-				inspector.FatContainerCmd = append(inspector.FatContainerCmd, overrides.Cmd...)
-			} else {
-				inspector.FatContainerCmd = append(inspector.FatContainerCmd, imageInspector.ImageInfo.Config.Cmd...)
-			}
-		}
-	}
-
-	emptyIdx := -1
-	for idx, val := range inspector.FatContainerCmd {
-		val = strings.TrimSpace(val)
-		if val != "" {
-			break
-		}
-
-		emptyIdx = idx
-	}
-
-	if emptyIdx > -1 {
-		inspector.FatContainerCmd = inspector.FatContainerCmd[emptyIdx+1:]
+		inspector.FatContainerCmd = BuildStartupCommand(
+			imageInspector.ImageInfo.Config.Entrypoint,
+			imageInspector.ImageInfo.Config.Cmd,
+			imageInspector.ImageInfo.Config.Shell,
+			overrides.ClearEntrypoint,
+			overrides.Entrypoint,
+			overrides.ClearCmd,
+			overrides.Cmd,
+		)
 	}
 
 	logger.Debugf("FatContainerCmd - %+v", inspector.FatContainerCmd)
@@ -495,6 +495,10 @@ func (i *Inspector) RunContainer() error {
 		containerCmd = append(containerCmd, "-log-format", i.LogFormat)
 	}
 
+	if i.DoEnableMondel {
+		containerCmd = append(containerCmd, "-n")
+	}
+
 	i.ContainerName = fmt.Sprintf(ContainerNamePat, os.Getpid(), time.Now().UTC().Format("20060102150405"))
 
 	labels := i.Overrides.Labels
@@ -629,6 +633,7 @@ func (i *Inspector) RunContainer() error {
 	if err != nil {
 		return err
 	}
+	// note: now need to cleanup the created container if there's an error
 
 	if i.ContainerName != containerInfo.Name {
 		logger.Debugf("Container name mismatch expected=%v got=%v", i.ContainerName, containerInfo.Name)
@@ -732,12 +737,17 @@ func (i *Inspector) RunContainer() error {
 		return err
 	}
 
-	errutil.FailWhen(i.ContainerInfo.NetworkSettings == nil, "slim: error => no network info")
+	if i.ContainerInfo.NetworkSettings == nil {
+		return fmt.Errorf("slim: error => no network info")
+	}
 
 	if hCfg := i.ContainerInfo.HostConfig; hCfg != nil && !i.isHostNetworked() {
 		logger.Debugf("container HostConfig.NetworkMode => %s len(ports)=%d",
 			hCfg.NetworkMode, len(i.ContainerInfo.NetworkSettings.Ports))
-		errutil.FailWhen(len(i.ContainerInfo.NetworkSettings.Ports) < len(commsExposedPorts), "slim: error => missing comms ports")
+
+		if len(i.ContainerInfo.NetworkSettings.Ports) < len(commsExposedPorts) {
+			return fmt.Errorf("slim: error => missing comms ports")
+		}
 	}
 
 	logger.Debugf("container NetworkSettings.Ports => %#v", i.ContainerInfo.NetworkSettings.Ports)
@@ -770,6 +780,8 @@ func (i *Inspector) RunContainer() error {
 		cmd.Excludes = pathMapKeys(i.ExcludePatterns)
 	}
 
+	cmd.ExcludeVarLockFiles = i.DoExcludeVarLockFiles
+
 	if len(i.PreservePaths) > 0 {
 		cmd.Preserves = i.PreservePaths
 	}
@@ -788,6 +800,10 @@ func (i *Inspector) RunContainer() error {
 		cmd.IncludeBins = pathMapKeys(i.IncludeBins)
 	}
 
+	if len(i.IncludeDirBinsList) > 0 {
+		cmd.IncludeDirBinsList = i.IncludeDirBinsList
+	}
+
 	if len(i.IncludeExes) > 0 {
 		cmd.IncludeExes = pathMapKeys(i.IncludeExes)
 	}
@@ -804,7 +820,9 @@ func (i *Inspector) RunContainer() error {
 	cmd.IncludeCertPKAll = i.DoIncludeCertPKAll
 	cmd.IncludeCertPKDirs = i.DoIncludeCertPKDirs
 	cmd.IncludeNew = i.DoIncludeNew
+	cmd.IncludeSSHClient = i.DoIncludeSSHClient
 	cmd.IncludeOSLibsNet = i.DoIncludeOSLibsNet
+	cmd.IncludeZoneInfo = i.DoIncludeZoneInfo
 
 	if runAsUser != "" {
 		cmd.AppUser = runAsUser
@@ -895,6 +913,8 @@ func (i *Inspector) RunContainer() error {
 					})
 			}
 
+			//not returning an error, exiting, need to clean up the container
+			i.ShutdownContainer(true)
 			i.xc.Exit(-124)
 		}
 
@@ -903,9 +923,10 @@ func (i *Inspector) RunContainer() error {
 				i.xc.Out.Info("event.startmonitor.done",
 					ovars{
 						"status": "received.unexpected",
-						"data":   fmt.Sprintf("%+v", evt),
+						"data":   jsonutil.ToString(evt),
 					})
 			}
+
 			return event.ErrUnexpectedEvent
 		}
 	}
@@ -1132,7 +1153,12 @@ func (i *Inspector) ShowContainerLogs() {
 }
 
 // ShutdownContainer terminates the container inspector instance execution
-func (i *Inspector) ShutdownContainer() error {
+func (i *Inspector) ShutdownContainer(terminateOnly bool) error {
+	if i.ContainerID == "" {
+		//no container to shutdown...
+		return nil
+	}
+
 	if i.isDone.IsOn() {
 		return nil
 	}
@@ -1162,59 +1188,79 @@ func (i *Inspector) ShutdownContainer() error {
 		}
 
 		if err := i.APIClient.RemoveContainer(removeOption); err != nil {
-			logger.Info("error removing container =>", err)
+			logger.Infof("error removing container ('%v')... terminating container", err)
+			_ = i.APIClient.KillContainer(dockerapi.KillContainerOptions{ID: i.ContainerID})
 		}
 	}()
 
-	if !i.DoUseLocalMounts {
-		deleteOrig := true
-		if i.DoKeepTmpArtifacts {
-			deleteOrig = false
-		}
-
-		reportLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, ReportArtifactTar)
-		reportRemotePath := filepath.Join(app.DefaultArtifactsDirPath, report.DefaultContainerReportFileName)
-		err := dockerutil.CopyFromContainer(i.APIClient, i.ContainerID, reportRemotePath, reportLocalPath, true, deleteOrig)
-		if err != nil {
-			logger.WithError(err).WithField("container", i.ContainerID).Error("dockerutil.CopyFromContainer")
-			//can't call errutil.FailOn() because we won't cleanup the target container
-			return err
-		}
-
-		/*
-			//ALTERNATIVE WAY TO XFER THE FILE ARTIFACTS
-			filesOutLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, FileArtifactsArchiveTar)
-			filesTarRemotePath := filepath.Join(app.DefaultArtifactsDirPath, fileArtifactsTar)
-			err = dockerutil.CopyFromContainer(i.APIClient,
-				i.ContainerID,
-				filesTarRemotePath,
-				filesOutLocalPath,
-				true,
-				false) //make it 'true' once tested/debugged
-			if err != nil {
-				errutil.FailOn(err)
+	if !terminateOnly {
+		if !i.DoUseLocalMounts {
+			deleteOrig := true
+			if i.DoKeepTmpArtifacts {
+				deleteOrig = false
 			}
-		*/
 
-		filesOutLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, FileArtifactsOutTar)
-		filesRemotePath := filepath.Join(app.DefaultArtifactsDirPath, app.ArtifactFilesDirName)
-		err = dockerutil.CopyFromContainer(i.APIClient, i.ContainerID, filesRemotePath, filesOutLocalPath, false, false)
-		if err != nil {
-			logger.WithError(err).WithField("container", i.ContainerID).Error("dockerutil.CopyFromContainer")
-			//can't call errutil.FailOn() because we won't cleanup the target container
-			return err
-		}
+			//copy the container report
+			reportLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, ReportArtifactTar)
+			reportRemotePath := filepath.Join(app.DefaultArtifactsDirPath, report.DefaultContainerReportFileName)
+			err := dockerutil.CopyFromContainer(i.APIClient, i.ContainerID, reportRemotePath, reportLocalPath, true, deleteOrig)
+			if err != nil {
+				logger.WithError(err).WithField("container", i.ContainerID).Error("dockerutil.CopyFromContainer")
+				//can't call errutil.FailOn() because we won't cleanup the target container
+				return err
+			}
 
-		//NOTE: possible enhancement (if the original filemode bits still get lost)
-		//(alternative to archiving files in the container to preserve filemodes)
-		//Rewrite the filemode bits using the data from creport.json,
-		//but creport.json also needs to be enhanced to use
-		//octal filemodes for the file records
-		err = dockerutil.PrepareContainerDataArchive(filesOutLocalPath, fileArtifactsTar, app.ArtifactFilesDirName+"/", deleteOrig)
-		if err != nil {
-			logger.WithError(err).WithField("container", i.ContainerID).Error("dockerutil.PrepareContainerDataArchive")
-			//can't call errutil.FailOn() because we won't cleanup the target container
-			return err
+			if i.DoEnableMondel {
+				//copy the monitor data event log (if available)
+				mondelLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, MondelArtifactTar)
+				mondelRemotePath := filepath.Join(app.DefaultArtifactsDirPath, report.DefaultMonDelFileName)
+				err = dockerutil.CopyFromContainer(i.APIClient, i.ContainerID, mondelRemotePath, mondelLocalPath, true, deleteOrig)
+				if err != nil {
+					//not a failure because the log might not be there (just log it)
+					logger.WithFields(log.Fields{
+						"artifact.type": "mondel",
+						"local.path":    mondelLocalPath,
+						"remote.path":   mondelRemotePath,
+						"err":           err,
+					}).Debug("dockerutil.CopyFromContainer")
+				}
+			}
+
+			/*
+				//ALTERNATIVE WAY TO XFER THE FILE ARTIFACTS
+				filesOutLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, FileArtifactsArchiveTar)
+				filesTarRemotePath := filepath.Join(app.DefaultArtifactsDirPath, fileArtifactsTar)
+				err = dockerutil.CopyFromContainer(i.APIClient,
+					i.ContainerID,
+					filesTarRemotePath,
+					filesOutLocalPath,
+					true,
+					false) //make it 'true' once tested/debugged
+				if err != nil {
+					errutil.FailOn(err)
+				}
+			*/
+
+			filesOutLocalPath := filepath.Join(i.LocalVolumePath, ArtifactsDir, FileArtifactsOutTar)
+			filesRemotePath := filepath.Join(app.DefaultArtifactsDirPath, app.ArtifactFilesDirName)
+			err = dockerutil.CopyFromContainer(i.APIClient, i.ContainerID, filesRemotePath, filesOutLocalPath, false, false)
+			if err != nil {
+				logger.WithError(err).WithField("container", i.ContainerID).Error("dockerutil.CopyFromContainer")
+				//can't call errutil.FailOn() because we won't cleanup the target container
+				return err
+			}
+
+			//NOTE: possible enhancement (if the original filemode bits still get lost)
+			//(alternative to archiving files in the container to preserve filemodes)
+			//Rewrite the filemode bits using the data from creport.json,
+			//but creport.json also needs to be enhanced to use
+			//octal filemodes for the file records
+			err = dockerutil.PrepareContainerDataArchive(filesOutLocalPath, fileArtifactsTar, app.ArtifactFilesDirName+"/", deleteOrig)
+			if err != nil {
+				logger.WithError(err).WithField("container", i.ContainerID).Error("dockerutil.PrepareContainerDataArchive")
+				//can't call errutil.FailOn() because we won't cleanup the target container
+				return err
+			}
 		}
 	}
 
