@@ -11,21 +11,21 @@ import (
 	"sync"
 	"time"
 
-	"github.com/docker-slim/docker-slim/pkg/app"
-	"github.com/docker-slim/docker-slim/pkg/app/master/config"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/image"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/ipc"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/sensor"
-	"github.com/docker-slim/docker-slim/pkg/app/master/kubernetes"
-	"github.com/docker-slim/docker-slim/pkg/app/master/security/apparmor"
-	"github.com/docker-slim/docker-slim/pkg/app/master/security/seccomp"
-	"github.com/docker-slim/docker-slim/pkg/ipc/channel"
-	"github.com/docker-slim/docker-slim/pkg/ipc/command"
-	"github.com/docker-slim/docker-slim/pkg/ipc/event"
-	"github.com/docker-slim/docker-slim/pkg/report"
-	"github.com/docker-slim/docker-slim/pkg/util/errutil"
-	"github.com/docker-slim/docker-slim/pkg/util/fsutil"
-	v "github.com/docker-slim/docker-slim/pkg/version"
+	"github.com/slimtoolkit/slim/pkg/app"
+	"github.com/slimtoolkit/slim/pkg/app/master/config"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/image"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/ipc"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/sensor"
+	"github.com/slimtoolkit/slim/pkg/app/master/kubernetes"
+	"github.com/slimtoolkit/slim/pkg/app/master/security/apparmor"
+	"github.com/slimtoolkit/slim/pkg/app/master/security/seccomp"
+	"github.com/slimtoolkit/slim/pkg/ipc/channel"
+	"github.com/slimtoolkit/slim/pkg/ipc/command"
+	"github.com/slimtoolkit/slim/pkg/ipc/event"
+	"github.com/slimtoolkit/slim/pkg/report"
+	"github.com/slimtoolkit/slim/pkg/util/errutil"
+	"github.com/slimtoolkit/slim/pkg/util/fsutil"
+	v "github.com/slimtoolkit/slim/pkg/version"
 
 	dockerapi "github.com/fsouza/go-dockerclient"
 	log "github.com/sirupsen/logrus"
@@ -662,6 +662,8 @@ func (i *Inspector) sensorCommandStart() error {
 					"status": "received.unexpected",
 					"data":   fmt.Sprintf("%+v", evt),
 				})
+
+			//TODO: dump temp container logs
 			return event.ErrUnexpectedEvent
 		}
 	}
