@@ -13,10 +13,11 @@ import (
 	"github.com/google/uuid"
 	log "github.com/sirupsen/logrus"
 
-	"github.com/docker-slim/docker-slim/pkg/ipc/event"
-	"github.com/docker-slim/docker-slim/pkg/report"
-	testsensor "github.com/docker-slim/docker-slim/pkg/test/e2e/sensor"
-	testutil "github.com/docker-slim/docker-slim/pkg/test/util"
+	"github.com/slimtoolkit/slim/pkg/app/sensor/standalone/control"
+	"github.com/slimtoolkit/slim/pkg/ipc/event"
+	"github.com/slimtoolkit/slim/pkg/report"
+	testsensor "github.com/slimtoolkit/slim/pkg/test/e2e/sensor"
+	testutil "github.com/slimtoolkit/slim/pkg/test/util"
 )
 
 const (
@@ -29,8 +30,7 @@ var (
 		"sensor: ver=",
 		"sensor: creating monitors...",
 		"sensor: starting monitors...",
-		"sensor: monitor - saving report",
-		"sensor: done!",
+		"sensor: run finished succesfully",
 	}
 
 	sensorLifecycleHookSequence = []string{
@@ -544,7 +544,7 @@ func TestArchiveArtifacts_SensorFailure_NoCaps(t *testing.T) {
 		"sensor: creating monitors...",
 		"sensor: starting monitors...",
 		"sensor: composite monitor - FAN failed to start running", // <-- failure!
-		"sensor: done!",
+		"sensor: run finished with error",
 	}...)
 
 	sensor.AssertArtifactsArchiveContains(t, ctx,
@@ -558,3 +558,115 @@ func TestArchiveArtifacts_SensorFailure_NoRoot(t *testing.T) {
 	// It's a fairly common failure scenario.
 	t.Skip("Implement me!")
 }
+
+func TestStopSignal_ForceKill(t *testing.T) {
+	runID := newTestRun(t)
+	ctx := context.Background()
+
+	wrongStopSignal := syscall.SIGUSR1 // This signal isn't going to make Nginx exit.
+	sensor := testsensor.NewSensorOrFail(
+		t, ctx, t.TempDir(), runID, imageSimpleService,
+		testsensor.WithStopSignal(wrongStopSignal), // Emulate misconfiguration.
+	)
+	defer sensor.Cleanup(t, ctx)
+
+	sensor.StartStandaloneOrFail(t, ctx, nil)
+	go testutil.Delayed(ctx, 5*time.Second, func() {
+		// However, the sensor will terminate the target app
+		// anyway, when the grace period after receiving the
+		// stop signal is over.
+		sensor.SignalOrFail(t, ctx, wrongStopSignal)
+	})
+	sensor.WaitOrFail(t, ctx)
+
+	sensor.AssertSensorLogsContain(t, ctx, sensorFullLifecycleSequence...)
+	sensor.AssertTargetAppLogsContain(t, ctx,
+		"nginx/1.21",
+		"start worker processes",
+		"sensor: stop signal was sent to target app - starting grace period",
+		"sensor: grace timeout expired - SIGKILL goes to target app",
+	)
+
+	sensor.DownloadArtifactsOrFail(t, ctx)
+	sensor.AssertReportIncludesFiles(t,
+		"/etc/nginx/nginx.conf",
+		"/etc/nginx/conf.d/default.conf",
+		"/var/cache/nginx",
+		"/var/run",
+		// Because the target app was terminated by SIGKILL,
+		// the pid file is not cleaned up.
+		"/run/nginx.pid",
+	)
+	sensor.AssertReportNotIncludesFiles(t,
+		"/bin/bash",
+		"/bin/cat",
+		"/etc/apt/sources.list",
+	)
+}
+
+func TestControlCommands_StopTargetApp(t *testing.T) {
+	runID := newTestRun(t)
+	ctx := context.Background()
+
+	sensor := testsensor.NewSensorOrFail(t, ctx, t.TempDir(), runID, imageSimpleService)
+	defer sensor.Cleanup(t, ctx)
+
+	sensor.StartStandaloneOrFail(t, ctx, nil)
+
+	go testutil.Delayed(ctx, 5*time.Second, func() {
+		sensor.ExecuteControlCommandOrFail(t, ctx, control.StopTargetAppCommand)
+		sensor.WaitForEventOrFail(t, ctx, event.StopMonitorDone)
+		sensor.WaitForEventOrFail(t, ctx, event.ShutdownSensorDone)
+
+		// In the real world, there might be some (long) time between
+		// the stop command and the target app signalling - maybe
+		// we need to simulate that here?
+		sensor.SignalOrFail(t, ctx, syscall.SIGQUIT)
+	})
+
+	sensor.WaitOrFail(t, ctx)
+
+	sensor.AssertSensorLogsContain(t, ctx, sensorFullLifecycleSequence...)
+}
+
+func TestEnableMondel(t *testing.T) {
+	runID := newTestRun(t)
+	ctx := context.Background()
+
+	sensor := testsensor.NewSensorOrFail(
+		t, ctx, t.TempDir(), runID, imageSimpleService,
+		testsensor.WithEnableMondel(),
+	)
+	defer sensor.Cleanup(t, ctx)
+
+	sensor.StartStandaloneOrFail(t, ctx, nil)
+	go testutil.Delayed(ctx, 5*time.Second, func() {
+		sensor.SignalOrFail(t, ctx, syscall.SIGTERM)
+	})
+	sensor.WaitOrFail(t, ctx)
+
+	sensor.AssertSensorLogsContain(t, ctx, sensorFullLifecycleSequence...)
+
+	sensor.DownloadArtifactsOrFail(t, ctx)
+
+	sensor.AssertMondelIncludesFiles(t,
+		"/etc/nginx/nginx.conf",
+		"/etc/nginx/conf.d/default.conf",
+
+		// TODO: investigate why these files are not included in the mondel (but are in the creport).
+		// "/bin/sh",
+		// "/var/cache/nginx",
+		// "/var/run",
+	)
+	sensor.AssertMondelNotIncludesFiles(t,
+		"/bin/bash",
+		"/bin/cat",
+		"/etc/apt/sources.list",
+
+		// TODO: investigate why this file is included in the mondel (but not in the creport).
+		// "/run/nginx.pid",
+	)
+
+	// Uncomment when the mondel and creport file sets are synced.
+	// sensor.AssertReportAndMondelFileListsMatch(t)
+}
