@@ -16,8 +16,8 @@ import (
 	dockerapi "github.com/fsouza/go-dockerclient"
 	log "github.com/sirupsen/logrus"
 
-	"github.com/docker-slim/docker-slim/pkg/docker/dockerclient"
-	"github.com/docker-slim/docker-slim/pkg/util/fsutil"
+	"github.com/slimtoolkit/slim/pkg/docker/dockerclient"
+	"github.com/slimtoolkit/slim/pkg/util/fsutil"
 )
 
 var (
@@ -63,20 +63,30 @@ func ImageToIdentity(info *dockerapi.Image) *ImageIdentity {
 		RepoDigests: info.RepoDigests,
 	}
 
+	uniqueTags := map[string]struct{}{}
 	for _, tag := range result.RepoTags {
 		parts := strings.Split(tag, ":")
 		if len(parts) == 2 {
-			result.ShortTags = append(result.ShortTags, parts[1])
+			uniqueTags[parts[1]] = struct{}{}
 		}
 	}
 
+	for k := range uniqueTags {
+		result.ShortTags = append(result.ShortTags, k)
+	}
+
+	uniqueDigests := map[string]struct{}{}
 	for _, digest := range result.RepoDigests {
 		parts := strings.Split(digest, "@")
 		if len(parts) == 2 {
-			result.ShortDigests = append(result.ShortDigests, parts[1])
+			uniqueDigests[parts[1]] = struct{}{}
 		}
 	}
 
+	for k := range uniqueDigests {
+		result.ShortDigests = append(result.ShortDigests, k)
+	}
+
 	return result
 }
 
@@ -103,12 +113,16 @@ func HasImage(dclient *dockerapi.Client, imageRef string) (*ImageIdentity, error
 
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return nil, err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return nil, fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.HasImage(%s): dockerapi.NewClient() error = %v", imageRef, err)
 			return nil, err
@@ -138,12 +152,16 @@ func ListImages(dclient *dockerapi.Client, imageNameFilter string) (map[string]B
 	// * <- all image names with no/default namespace. note that no images with namespaces will be returned
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return nil, err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return nil, fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.ListImages(%s): dockerapi.NewClient() error = %v", imageNameFilter, err)
 			return nil, err
@@ -192,14 +210,18 @@ func ListImages(dclient *dockerapi.Client, imageNameFilter string) (map[string]B
 
 func BuildEmptyImage(dclient *dockerapi.Client) error {
 	//TODO: use the 'internal' build engine that doesn't need Docker
-	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		//note: shouldn't use ":=" here to avoid var shadowing
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.BuildEmptyImage: dockerapi.NewClient() error = %v", err)
 			return err
@@ -248,12 +270,16 @@ func SaveImage(dclient *dockerapi.Client, imageRef, local string, extract, remov
 
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.SaveImage: dockerapi.NewClient() error = %v", err)
 			return err
@@ -310,8 +336,8 @@ func SaveImage(dclient *dockerapi.Client, imageRef, local string, extract, remov
 
 		}
 
-		tarOptions.IDMap.UIDMaps = arc.IDMapping.UIDs()
-		tarOptions.IDMap.GIDMaps = arc.IDMapping.GIDs()
+		tarOptions.IDMap.UIDMaps = arc.IDMapping.UIDMaps
+		tarOptions.IDMap.GIDMaps = arc.IDMapping.GIDMaps
 
 		err = arc.Untar(afile, dstDir, tarOptions)
 		if err != nil {
@@ -337,12 +363,16 @@ func HasVolume(dclient *dockerapi.Client, name string) error {
 
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.HasVolume: dockerapi.NewClient() error = %v", err)
 			return err
@@ -378,14 +408,18 @@ func DeleteVolume(dclient *dockerapi.Client, name string) error {
 		return ErrBadParam
 	}
 
-	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		//note: shouldn't use ":=" here to avoid var shadowing
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.DeleteVolume: dockerapi.NewClient() error = %v", err)
 			return err
@@ -417,12 +451,16 @@ func CopyToVolume(
 	dstTargetDir string) error {
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.CopyToVolume: dockerapi.NewClient() error = %v", err)
 			return err
@@ -582,12 +620,16 @@ func CreateVolumeWithData(
 
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.CreateVolumeWithData: dockerapi.NewClient() error = %v", err)
 			return err
@@ -621,12 +663,16 @@ func CopyFromContainer(dclient *dockerapi.Client, containerID, remote, local str
 
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.CopyFromContainer: dockerapi.NewClient() error = %v", err)
 			return err
@@ -669,8 +715,8 @@ func CopyFromContainer(dclient *dockerapi.Client, containerID, remote, local str
 			//GIDMaps:  arc.IDMapping.GIDs(),
 		}
 
-		tarOptions.IDMap.UIDMaps = arc.IDMapping.UIDs()
-		tarOptions.IDMap.GIDMaps = arc.IDMapping.GIDs()
+		tarOptions.IDMap.UIDMaps = arc.IDMapping.UIDMaps
+		tarOptions.IDMap.GIDMaps = arc.IDMapping.GIDMaps
 
 		err = arc.Untar(afile, dstDir, tarOptions)
 		if err != nil {
@@ -770,12 +816,16 @@ func PrepareContainerDataArchive(fullPath, newName, removePrefix string, removeO
 func ListNetworks(dclient *dockerapi.Client, nameFilter string) ([]string, error) {
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return nil, err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return nil, fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.ListNetworks(%s): dockerapi.NewClient() error = %v", nameFilter, err)
 			return nil, err
@@ -805,12 +855,16 @@ func ListNetworks(dclient *dockerapi.Client, nameFilter string) ([]string, error
 func ListVolumes(dclient *dockerapi.Client, nameFilter string) ([]string, error) {
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return nil, err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return nil, fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.ListVolumes(%s): dockerapi.NewClient() error = %v", nameFilter, err)
 			return nil, err
@@ -863,12 +917,16 @@ type BasicContainerProps struct {
 func ListContainers(dclient *dockerapi.Client, nameFilter string, all bool) (map[string]BasicContainerProps, error) {
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return nil, err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return nil, fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.ListContainers(%s): dockerapi.NewClient() error = %v", nameFilter, err)
 			return nil, err
@@ -919,12 +977,16 @@ func ListContainers(dclient *dockerapi.Client, nameFilter string, all bool) (map
 func GetContainerLogs(dclient *dockerapi.Client, containerID string, rawTerminal bool) (string, string, error) {
 	var err error
 	if dclient == nil {
-		unixSocketAddr := dockerclient.GetUnixSocketAddr()
-		if unixSocketAddr == "" {
+		socketInfo, err := dockerclient.GetUnixSocketAddr()
+		if err != nil {
+			return "", "", err
+		}
+
+		if socketInfo == nil || socketInfo.Address == "" {
 			return "", "", fmt.Errorf("no unix socket found")
 		}
 
-		dclient, err = dockerapi.NewClient(unixSocketAddr)
+		dclient, err = dockerapi.NewClient(socketInfo.Address)
 		if err != nil {
 			log.Errorf("dockerutil.GetContainerLogs(%s): dockerapi.NewClient() error = %v", containerID, err)
 			return "", "", err
