@@ -11,6 +11,7 @@ import (
 	"os"
 	"os/exec"
 	"path"
+	"path/filepath"
 	"runtime"
 	"strconv"
 	"strings"
@@ -20,10 +21,11 @@ import (
 	log "github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 
-	"github.com/docker-slim/docker-slim/pkg/errors"
-	"github.com/docker-slim/docker-slim/pkg/launcher"
-	"github.com/docker-slim/docker-slim/pkg/report"
-	"github.com/docker-slim/docker-slim/pkg/system"
+	"github.com/slimtoolkit/slim/pkg/errors"
+	"github.com/slimtoolkit/slim/pkg/launcher"
+	"github.com/slimtoolkit/slim/pkg/mondel"
+	"github.com/slimtoolkit/slim/pkg/report"
+	"github.com/slimtoolkit/slim/pkg/system"
 )
 
 type AppState string
@@ -34,8 +36,13 @@ const (
 	AppDone    AppState = "app.done"
 )
 
+const (
+	cwdFD = unix.AT_FDCWD // AT_FDCWD = -0x64
+)
+
 func Run(
 	ctx context.Context,
+	del mondel.Publisher,
 	runOpt AppRunOpt,
 	// TODO(ivan): includeNew & origPaths logic should be applied at the artifact dumping stage.
 	includeNew bool,
@@ -51,7 +58,7 @@ func Run(
 	logger.Debug("call")
 	defer logger.Debug("exit")
 
-	app, err := newApp(ctx, runOpt, includeNew, origPaths, signalCh, errorCh)
+	app, err := newApp(ctx, del, runOpt, includeNew, origPaths, signalCh, errorCh)
 	if err != nil {
 		app.StateCh <- AppFailed
 		return nil, err
@@ -122,6 +129,8 @@ type App struct {
 	User      string
 	RunAsUser bool
 
+	del mondel.Publisher
+
 	appStdout io.Writer
 	appStderr io.Writer
 
@@ -162,6 +171,7 @@ func (a *App) PGID() int {
 const eventBufSize = 2000
 
 type syscallEvent struct {
+	returned  bool
 	pid       int
 	callNum   uint32
 	retVal    uint64
@@ -170,6 +180,7 @@ type syscallEvent struct {
 
 func newApp(
 	ctx context.Context,
+	del mondel.Publisher,
 	runOpt AppRunOpt,
 	includeNew bool,
 	origPaths map[string]struct{},
@@ -185,6 +196,7 @@ func newApp(
 
 	a := App{
 		ctx:       ctx,
+		del:       del,
 		Cmd:       runOpt.Cmd,
 		Args:      runOpt.Args,
 		WorkDir:   runOpt.WorkDir,
@@ -251,16 +263,17 @@ func (app *App) processSyscallActivity(e *syscallEvent) {
 
 func (app *App) processFileActivity(e *syscallEvent) {
 	if e.pathParam != "" {
+		logger := app.logger.WithField("op", "processFileActivity")
 		p, found := syscallProcessors[int(e.callNum)]
 		if !found {
-			app.logger.WithField("op", "processFileActivity").Debugf("no syscall processor - %#v", e)
+			logger.Debugf("no syscall processor - %#v", e)
 			//shouldn't happen
 			return
 		}
 
 		if (p.SyscallType() == CheckFileType ||
 			p.SyscallType() == OpenFileType) &&
-			!p.FailedReturnStatus(e.retVal) {
+			p.OKReturnStatus(e.retVal) {
 			//todo: filter "/proc/", "/sys/", "/dev/" externally
 			if e.pathParam != "." &&
 				e.pathParam != "/proc" &&
@@ -291,6 +304,78 @@ func (app *App) processFileActivity(e *syscallEvent) {
 
 					app.fsActivity[e.pathParam] = fsa
 				}
+
+				if app.del != nil {
+					//NOTE:
+					//not capturing the 'dirfd' syscall params necessary
+					//to reconstruct relative paths for some syscalls (todo: improve later)
+					delEvent := &report.MonitorDataEvent{
+						Source:   report.MDESourcePT,
+						Type:     report.MDETypeArtifact,
+						Pid:      int32(e.pid),
+						Artifact: e.pathParam, //note: might not be full path
+						OpNum:    e.callNum,
+						Op:       p.SyscallName(),
+					}
+
+					switch p.SyscallType() {
+					case CheckFileType:
+						delEvent.OpType = report.OpTypeCheck
+					case OpenFileType:
+						delEvent.OpType = report.OpTypeRead
+					}
+
+					if err := app.del.Publish(delEvent); err != nil {
+						logger.Errorf(
+							"mondel publish event failed - source=%v type=%v: %v",
+							delEvent.Source, delEvent.Type, err,
+						)
+					}
+				}
+			}
+		}
+
+		if p.SyscallType() == ExecType &&
+			(!e.returned || //need to catch exec calls that haven't completed yet
+				(e.returned && p.OKReturnStatus(e.retVal))) {
+			if fsa, ok := app.fsActivity[e.pathParam]; ok {
+				fsa.OpsAll++
+				fsa.Pids[e.pid] = struct{}{}
+				fsa.Syscalls[int(e.callNum)] = struct{}{}
+			} else {
+				fsa := &report.FSActivityInfo{
+					OpsAll:       1,
+					OpsCheckFile: 0,
+					Pids:         map[int]struct{}{},
+					Syscalls:     map[int]struct{}{},
+				}
+
+				fsa.Pids[e.pid] = struct{}{}
+				fsa.Syscalls[int(e.callNum)] = struct{}{}
+
+				app.fsActivity[e.pathParam] = fsa
+			}
+
+			if app.del != nil {
+				//NOTE:
+				//not capturing the 'dirfd' syscall params necessary
+				//to reconstruct relative paths for some syscalls (todo: improve later)
+				delEvent := &report.MonitorDataEvent{
+					Source:   report.MDESourcePT,
+					Type:     report.MDETypeArtifact,
+					Pid:      int32(e.pid),
+					Artifact: e.pathParam, //note: might not be full path
+					OpNum:    e.callNum,
+					Op:       p.SyscallName(),
+					OpType:   report.OpTypeExec,
+				}
+
+				if err := app.del.Publish(delEvent); err != nil {
+					logger.Errorf(
+						"mondel publish event failed - source=%v type=%v op_type=%v: %v",
+						delEvent.Source, delEvent.Type, delEvent.OpType, err,
+					)
+				}
 			}
 		}
 	}
@@ -610,7 +695,7 @@ func (app *App) collect() {
 					// We'd been observing this behavior a lot with short&fast Go programs,
 					// and regular `strace -f ./app` would produce similar results.
 					// Sending this error event back to the master process would make
-					// the `docker-slim build` command fail with the exit code -124.
+					// the `slim build` command fail with the exit code -124.
 					logger.Debugf("trace syscall - (likely) tracee terminated pid=%v sig=%v error - %v (errno=%d)", callPid, callSig, err, err.(syscall.Errno))
 				} else {
 					logger.Errorf("trace syscall pid=%v sig=%v error - %v (errno=%d)", callPid, callSig, err, err.(syscall.Errno))
@@ -741,7 +826,9 @@ func (app *App) collect() {
 				}
 
 				if genEvent {
+					//not waiting for the return call for the 'exec' syscalls (mostly)
 					evt := syscallEvent{
+						returned:  false,
 						pid:       wpid,
 						callNum:   uint32(cstate.callNum),
 						pathParam: cstate.pathParam,
@@ -752,7 +839,7 @@ func (app *App) collect() {
 					select {
 					case app.eventCh <- evt:
 					default:
-						logger.Debugf("[%d/%d]: event.onSyscall - wpid=%v app.eventCh send error (evt=%#v)",
+						logger.Warnf("[%d/%d]: event.onSyscall - wpid=%v app.eventCh send error (evt=%#v)",
 							app.cmd.Process.Pid, app.pgid, wpid, evt)
 					}
 				}
@@ -766,6 +853,7 @@ func (app *App) collect() {
 
 			if cstate.gotCallNum && cstate.gotRetVal {
 				evt := syscallEvent{
+					returned:  true,
 					pid:       wpid,
 					callNum:   uint32(cstate.callNum),
 					retVal:    cstate.retVal,
@@ -786,7 +874,7 @@ func (app *App) collect() {
 					select {
 					case app.eventCh <- evt:
 					default:
-						logger.Debugf("[%d/%d]: wpid=%v app.eventCh send error (evt=%#v)",
+						logger.Warnf("[%d/%d]: wpid=%v app.eventCh send error (evt=%#v)",
 							app.cmd.Process.Pid, app.pgid, wpid, evt)
 					}
 				}
@@ -910,8 +998,8 @@ func onSyscall(pid int, cstate *syscallState) (bool, error) {
 
 	if processor, found := syscallProcessors[int(cstate.callNum)]; found && processor != nil {
 		processor.OnCall(pid, regs, cstate)
-		genEvent := processor.EventOnCall()
-		return genEvent, nil
+		doGenEvent := processor.EventOnCall()
+		return doGenEvent, nil
 	}
 
 	return false, nil
@@ -936,6 +1024,35 @@ func onSyscallReturn(pid int, cstate *syscallState) error {
 
 ///////////////////////////////////
 
+func fdName(fd int) string {
+	switch fd {
+	case 0:
+		return "STDIN"
+	case 1:
+		return "STDOUT"
+	case 2:
+		return "STDERR"
+	case cwdFD:
+		return "AT_FDCWD"
+	}
+
+	return ""
+}
+
+func getIntVal(ptr uint64) int {
+	return int(int32(ptr))
+}
+
+func errnoName(ptr uint64) string {
+	val := int(int32(ptr))
+	if val >= 0 {
+		return ""
+	}
+
+	val *= -1
+	return unix.ErrnoName(syscall.Errno(val))
+}
+
 func getIntParam(pid int, ptr uint64) int {
 	return int(int32(ptr))
 }
@@ -983,6 +1100,8 @@ type SyscallProcessor interface {
 	OnReturn(pid int, regs syscall.PtraceRegs, cstate *syscallState)
 	FailedCall(cstate *syscallState) bool
 	FailedReturnStatus(retVal uint64) bool
+	OKCall(cstate *syscallState) bool
+	OKReturnStatus(retVal uint64) bool
 }
 
 type StringParamPos int
@@ -1019,7 +1138,7 @@ func (ref *syscallProcessorCore) SyscallName() string {
 func (ref *syscallProcessorCore) OnCall(pid int, regs syscall.PtraceRegs, cstate *syscallState) {
 	pth := ""
 	dir := ""
-	cwd, _ := os.Readlink(fmt.Sprintf("/proc/%d/cwd", pid))
+	var fd int
 
 	switch ref.StringParam {
 	case SPPNo:
@@ -1028,34 +1147,52 @@ func (ref *syscallProcessorCore) OnCall(pid int, regs syscall.PtraceRegs, cstate
 	case SPPOne:
 		pth = getStringParam(pid, system.CallFirstParam(regs))
 	case SPPTwo:
-		fd := getIntParam(pid, system.CallFirstParam(regs))
-		if fd > 0 {
+		fd = getIntParam(pid, system.CallFirstParam(regs))
+		//cwdFD < 0 / (stdin=0/stdout=1/stderr=2)
+		if fd > 2 {
 			dir, _ = os.Readlink(fmt.Sprintf("/proc/%d/fd/%d", pid, fd))
 		}
 		pth = getStringParam(pid, system.CallSecondParam(regs))
 	default:
 		panic("unreachable")
 	}
-	if len(pth) > 0 && pth[0] != '/' {
+
+	//fmt.Printf("[pid=%d][1]ptrace.SPC.OnCall[%v/%v](fd=[%x/%d/%s]/pth:'%s')\n",
+	//	pid, ref.Num, ref.Name, fd, fd, fdName(fd), pth)
+
+	if len(pth) == 0 || (len(pth) > 0 && pth[0] != '/') {
 		if dir != "" {
 			pth = path.Join(dir, pth)
-		} else if cwd != "" {
-			pth = path.Join(cwd, pth)
+		} else if fd == cwdFD {
+			//todo: track cwd with process/pid (so we don't need to look it up later for each call)
+			cwd, _ := os.Readlink(fmt.Sprintf("/proc/%d/cwd", pid))
+			if cwd != "" {
+				pth = path.Join(cwd, pth)
+			}
 		}
 	}
 
-	cstate.pathParam = pth
+	cstate.pathParam = filepath.Clean(pth)
+	//fmt.Printf("[pid=%d][2]ptrace.SPC.OnCall[%v/%v](fd=[%x/%d/%s]/pp:'%s')\n",
+	//	pid, ref.Num, ref.Name, fd, fd, fdName(fd), cstate.pathParam)
 }
 
 func (ref *syscallProcessorCore) OnReturn(pid int, regs syscall.PtraceRegs, cstate *syscallState) {
 	//for stat/check syscalls:
 	//-2 => -ENOENT (No such file or directory)
-	//TODO: get/use error code enums
+	intRetVal := getIntVal(cstate.retVal)
+	errnoStr := errnoName(cstate.retVal)
 
 	if cstate.pathParamErr == nil {
-		log.Tracef("checkFileSyscallProcessor.OnReturn: [%d] {%d}%s('%s') = %d", pid, cstate.callNum, ref.Name, cstate.pathParam, int(cstate.retVal))
+		//fmt.Printf("[pid=%d][3]ptrace.SPC.OnReturn[%v/%v](pp:'%s') = %x/%d/%s\n",
+		//	pid, cstate.callNum, ref.Name, cstate.pathParam, cstate.retVal, intRetVal, errnoStr)
+
+		log.Tracef("checkFileSyscallProcessor.OnReturn: [%d] {%d}%s('%s') = %x/%d/%s", pid, cstate.callNum, ref.Name, cstate.pathParam, cstate.retVal, intRetVal, errnoStr)
 	} else {
-		log.Debugf("checkFileSyscallProcessor.OnReturn: [%d] {%d}%s(<unknown>/'%s') = %d [pp err => %v]", pid, cstate.callNum, ref.Name, cstate.pathParam, int(cstate.retVal), cstate.pathParamErr)
+		log.Debugf("checkFileSyscallProcessor.OnReturn: [%d] {%d}%s(<unknown>/'%s') = %x/%d/%s [pp err => %v]", pid, cstate.callNum, ref.Name, cstate.pathParam, cstate.retVal, intRetVal, errnoStr, cstate.pathParamErr)
+
+		//fmt.Printf("[pid=%d][3err]ptrace.SPC.OnReturn[%v/%v](pp:'%s'/ppe: '%v') = %x/%d/%s\n",
+		//	pid, cstate.callNum, ref.Name, cstate.pathParam, cstate.pathParamErr, cstate.retVal, intRetVal, errnoStr)
 	}
 }
 
@@ -1071,6 +1208,14 @@ func (ref *checkFileSyscallProcessor) FailedReturnStatus(retVal uint64) bool {
 	return retVal != 0
 }
 
+func (ref *checkFileSyscallProcessor) OKCall(cstate *syscallState) bool {
+	return cstate.retVal == 0
+}
+
+func (ref *checkFileSyscallProcessor) OKReturnStatus(retVal uint64) bool {
+	return retVal == 0
+}
+
 func (ref *checkFileSyscallProcessor) EventOnCall() bool {
 	return false
 }
@@ -1080,11 +1225,27 @@ type openFileSyscallProcessor struct {
 }
 
 func (ref *openFileSyscallProcessor) FailedCall(cstate *syscallState) bool {
-	return cstate.retVal < 0
+	fd := getIntVal(cstate.retVal)
+	return fd < 0
 }
 
 func (ref *openFileSyscallProcessor) FailedReturnStatus(retVal uint64) bool {
-	return retVal < 0
+	fd := getIntVal(retVal)
+	return fd < 0
+}
+
+func (ref *openFileSyscallProcessor) OKCall(cstate *syscallState) bool {
+	fd := getIntVal(cstate.retVal)
+	//need to make sure it works for readlink which returns the number of read bytes
+	//(later: has its own processor)
+	return fd >= 0
+}
+
+func (ref *openFileSyscallProcessor) OKReturnStatus(retVal uint64) bool {
+	fd := getIntVal(retVal)
+	//need to make sure it works for readlink which returns the number of read bytes
+	//(later: has its own processor)
+	return fd >= 0
 }
 
 func (ref *openFileSyscallProcessor) EventOnCall() bool {
@@ -1096,11 +1257,23 @@ type execSyscallProcessor struct {
 }
 
 func (ref *execSyscallProcessor) FailedCall(cstate *syscallState) bool {
-	return cstate.retVal < 0
+	fd := getIntVal(cstate.retVal)
+	return fd < 0
 }
 
 func (ref *execSyscallProcessor) FailedReturnStatus(retVal uint64) bool {
-	return retVal < 0
+	fd := getIntVal(retVal)
+	return fd < 0
+}
+
+func (ref *execSyscallProcessor) OKCall(cstate *syscallState) bool {
+	fd := getIntVal(cstate.retVal)
+	return fd >= 0
+}
+
+func (ref *execSyscallProcessor) OKReturnStatus(retVal uint64) bool {
+	fd := getIntVal(retVal)
+	return fd >= 0
 }
 
 func (ref *execSyscallProcessor) EventOnCall() bool {
@@ -1111,38 +1284,6 @@ func (ref *execSyscallProcessor) EventOnCall() bool {
 var syscallProcessors = map[int]SyscallProcessor{}
 
 func init() {
-	//stat(const char *filename,struct stat *statbuf)
-	addSyscallProcessor(&checkFileSyscallProcessor{
-		syscallProcessorCore: &syscallProcessorCore{
-			Name:        "stat",
-			Type:        CheckFileType,
-			StringParam: SPPOne,
-		},
-	})
-	//lstat(fconst char *filename, struct stat *statbuf)
-	addSyscallProcessor(&checkFileSyscallProcessor{
-		syscallProcessorCore: &syscallProcessorCore{
-			Name:        "lstat",
-			Type:        CheckFileType,
-			StringParam: SPPOne,
-		},
-	})
-	//access(const char *filename, int mode)
-	addSyscallProcessor(&checkFileSyscallProcessor{
-		syscallProcessorCore: &syscallProcessorCore{
-			Name:        "access",
-			Type:        CheckFileType,
-			StringParam: SPPOne,
-		},
-	})
-	//statfs(const char *pathname, struct statfs *buf)
-	addSyscallProcessor(&checkFileSyscallProcessor{
-		syscallProcessorCore: &syscallProcessorCore{
-			Name:        "statfs",
-			Type:        CheckFileType,
-			StringParam: SPPOne,
-		},
-	})
 	//readlink(const char *path, char *buf, int bufsiz)
 	//on success, returns the number of bytes placed in buf. On error, returns -1
 	//reusing "open" processor because the call error return values are similar
@@ -1195,14 +1336,7 @@ func init() {
 			StringParam: SPPTwo,
 		},
 	})
-	//faccessat(int dfd, const char *filename, int mode)
-	addSyscallProcessor(&checkFileSyscallProcessor{
-		syscallProcessorCore: &syscallProcessorCore{
-			Name:        "faccessat",
-			Type:        CheckFileType,
-			StringParam: SPPTwo,
-		},
-	})
+
 	//openat(int dfd, const char *filename, int flags, int mode)
 	addSyscallProcessor(&openFileSyscallProcessor{
 		syscallProcessorCore: &syscallProcessorCore{
@@ -1219,6 +1353,49 @@ func init() {
 			StringParam: SPPTwo,
 		},
 	})
+
+	//"check" system calls with file paths:
+
+	//access(const char *filename, int mode)
+	addSyscallProcessor(&checkFileSyscallProcessor{
+		syscallProcessorCore: &syscallProcessorCore{
+			Name:        "access",
+			Type:        CheckFileType,
+			StringParam: SPPOne,
+		},
+	})
+	//faccessat(int dfd, const char *filename, int mode)
+	addSyscallProcessor(&checkFileSyscallProcessor{
+		syscallProcessorCore: &syscallProcessorCore{
+			Name:        "faccessat",
+			Type:        CheckFileType,
+			StringParam: SPPTwo,
+		},
+	})
+	//stat(const char *filename,struct stat *statbuf)
+	addSyscallProcessor(&checkFileSyscallProcessor{
+		syscallProcessorCore: &syscallProcessorCore{
+			Name:        "stat",
+			Type:        CheckFileType,
+			StringParam: SPPOne,
+		},
+	})
+	//lstat(fconst char *filename, struct stat *statbuf)
+	addSyscallProcessor(&checkFileSyscallProcessor{
+		syscallProcessorCore: &syscallProcessorCore{
+			Name:        "lstat",
+			Type:        CheckFileType,
+			StringParam: SPPOne,
+		},
+	})
+	//statfs(const char *pathname, struct statfs *buf)
+	addSyscallProcessor(&checkFileSyscallProcessor{
+		syscallProcessorCore: &syscallProcessorCore{
+			Name:        "statfs",
+			Type:        CheckFileType,
+			StringParam: SPPOne,
+		},
+	})
 	//statx(int dirfd, const char *pathname, int flags, unsigned int mask, struct statx *statxbuf)
 	//dirfd: AT_FDCWD
 	//flags: AT_EMPTY_PATH
@@ -1229,6 +1406,15 @@ func init() {
 			StringParam: SPPTwo,
 		},
 	})
+	//newfstatat/fstatat(int dirfd, const char *restrict pathname, struct stat *restrict statbuf, int flags)
+	//dirfd: AT_FDCWD
+	addSyscallProcessor(&checkFileSyscallProcessor{
+		syscallProcessorCore: &syscallProcessorCore{
+			Name:        "newfstatat",
+			Type:        CheckFileType,
+			StringParam: SPPTwo,
+		},
+	})
 
 	//execve(const char *pathname, char *const argv[], char *const envp[])
 	addSyscallProcessor(&execSyscallProcessor{
