@@ -9,13 +9,13 @@ import (
 
 	log "github.com/sirupsen/logrus"
 
-	"github.com/docker-slim/docker-slim/pkg/command"
-	"github.com/docker-slim/docker-slim/pkg/docker/dockerfile/reverse"
-	"github.com/docker-slim/docker-slim/pkg/docker/dockerimage"
-	"github.com/docker-slim/docker-slim/pkg/docker/linter/check"
-	"github.com/docker-slim/docker-slim/pkg/system"
-	"github.com/docker-slim/docker-slim/pkg/util/errutil"
-	"github.com/docker-slim/docker-slim/pkg/version"
+	"github.com/slimtoolkit/slim/pkg/command"
+	"github.com/slimtoolkit/slim/pkg/docker/dockerfile/reverse"
+	"github.com/slimtoolkit/slim/pkg/docker/dockerimage"
+	"github.com/slimtoolkit/slim/pkg/docker/linter/check"
+	"github.com/slimtoolkit/slim/pkg/system"
+	"github.com/slimtoolkit/slim/pkg/util/errutil"
+	"github.com/slimtoolkit/slim/pkg/version"
 )
 
 // DefaultFilename is the default name for the command report
@@ -117,6 +117,8 @@ type BuildCommand struct {
 	MinifiedImageSize      int64                `json:"minified_image_size"`
 	MinifiedImageSizeHuman string               `json:"minified_image_size_human"`
 	MinifiedImage          string               `json:"minified_image"`
+	MinifiedImageID        string               `json:"minified_image_id"`
+	MinifiedImageDigest    string               `json:"minified_image_digest"`
 	MinifiedImageHasData   bool                 `json:"minified_image_has_data"`
 	MinifiedBy             float64              `json:"minified_by"`
 	ArtifactLocation       string               `json:"artifact_location"`
@@ -149,20 +151,20 @@ type ProfileCommand struct {
 }
 
 // Output Version for 'xray'
-const OVXrayCommand = "1.2.2"
+const OVXrayCommand = "1.2.3"
 
 // XrayCommand is the 'xray' command report data
 type XrayCommand struct {
 	Command
-	TargetReference      string                      `json:"target_reference"`
-	SourceImage          ImageMetadata               `json:"source_image"`
-	ArtifactLocation     string                      `json:"artifact_location"`
-	ImageReport          *dockerimage.ImageReport    `json:"image_report,omitempty"`
-	ImageStack           []*reverse.ImageInfo        `json:"image_stack"`
-	ImageLayers          []*dockerimage.LayerReport  `json:"image_layers"`
-	ImageArchiveLocation string                      `json:"image_archive_location"`
-	RawImageManifest     *dockerimage.ManifestObject `json:"raw_image_manifest,omitempty"`
-	RawImageConfig       *dockerimage.ConfigObject   `json:"raw_image_config,omitempty"`
+	TargetReference      string                            `json:"target_reference"`
+	SourceImage          ImageMetadata                     `json:"source_image"`
+	ArtifactLocation     string                            `json:"artifact_location"`
+	ImageReport          *dockerimage.ImageReport          `json:"image_report,omitempty"`
+	ImageStack           []*reverse.ImageInfo              `json:"image_stack"`
+	ImageLayers          []*dockerimage.LayerReport        `json:"image_layers"`
+	ImageArchiveLocation string                            `json:"image_archive_location"`
+	RawImageManifest     *dockerimage.DockerManifestObject `json:"raw_image_manifest,omitempty"`
+	RawImageConfig       *dockerimage.ConfigObject         `json:"raw_image_config,omitempty"`
 }
 
 // Output Version for 'lint'
@@ -181,6 +183,14 @@ type LintCommand struct {
 	Errors          map[string]error         `json:"errors,omitempty"` //map[CHECK_ID]ERROR_INFO
 }
 
+// Output Version for 'images'
+const OVImagesCommand = "1.0"
+
+// ImagesCommand is the 'images' command report data
+type ImagesCommand struct {
+	Command
+}
+
 // Output Version for 'containerize'
 const OVContainerizeCommand = "1.0"
 
@@ -258,6 +268,15 @@ type RegistryCommand struct {
 	TargetReference string `json:"target_reference"`
 }
 
+// Output Version for 'vulnerability'
+const OVVulnerabilityCommand = "1.0"
+
+// VulnerabilityCommand is the 'vulnerability' command report data
+type VulnerabilityCommand struct {
+	Command
+	Operation string `json:"operation"`
+}
+
 func (cmd *Command) init(containerized bool) {
 	cmd.Containerized = containerized
 	cmd.Engine = version.Current()
@@ -330,6 +349,21 @@ func NewLintCommand(reportLocation string, containerized bool) *LintCommand {
 	return cmd
 }
 
+// NewImagesCommand creates a new 'images' command report
+func NewImagesCommand(reportLocation string, containerized bool) *ImagesCommand {
+	cmd := &ImagesCommand{
+		Command: Command{
+			reportLocation: reportLocation,
+			Version:        OVImagesCommand, //images command 'results' version (report and artifacts)
+			Type:           command.Images,
+			State:          command.StateUnknown,
+		},
+	}
+
+	cmd.Command.init(containerized)
+	return cmd
+}
+
 // NewContainerizeCommand creates a new 'containerize' command report
 func NewContainerizeCommand(reportLocation string, containerized bool) *ContainerizeCommand {
 	cmd := &ContainerizeCommand{
@@ -465,6 +499,21 @@ func NewRegistryCommand(reportLocation string, containerized bool) *RegistryComm
 	return cmd
 }
 
+// NewVulnerabilityCommand creates a new 'registry' command report
+func NewVulnerabilityCommand(reportLocation string, containerized bool) *VulnerabilityCommand {
+	cmd := &VulnerabilityCommand{
+		Command: Command{
+			reportLocation: reportLocation,
+			Version:        OVVulnerabilityCommand,
+			Type:           command.Vulnerability,
+			State:          command.StateUnknown,
+		},
+	}
+
+	cmd.Command.init(containerized)
+	return cmd
+}
+
 func (p *Command) ReportLocation() string {
 	return p.reportLocation
 }
