@@ -1,8 +1,8 @@
 package sensor
 
 import (
-	"github.com/docker-slim/docker-slim/pkg/app/master/commands"
-	"github.com/docker-slim/docker-slim/pkg/ipc/command"
+	mastercommand "github.com/slimtoolkit/slim/pkg/app/master/command"
+	"github.com/slimtoolkit/slim/pkg/ipc/command"
 )
 
 type StartMonitorOpt func(*command.StartMonitor)
@@ -48,7 +48,7 @@ func WithAppStderrToFile() StartMonitorOpt {
 
 func WithPreserves(path ...string) StartMonitorOpt {
 	return func(cmd *command.StartMonitor) {
-		cmd.Preserves = commands.ParsePaths(path)
+		cmd.Preserves = mastercommand.ParsePaths(path)
 	}
 }
 
