@@ -17,15 +17,17 @@ import (
 
 	dockerapi "github.com/fsouza/go-dockerclient"
 	log "github.com/sirupsen/logrus"
+	"golang.org/x/sys/unix"
 
-	"github.com/docker-slim/docker-slim/pkg/app"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/ipc"
-	"github.com/docker-slim/docker-slim/pkg/app/master/inspectors/sensor"
-	"github.com/docker-slim/docker-slim/pkg/ipc/channel"
-	"github.com/docker-slim/docker-slim/pkg/ipc/command"
-	"github.com/docker-slim/docker-slim/pkg/ipc/event"
-	"github.com/docker-slim/docker-slim/pkg/report"
-	"github.com/docker-slim/docker-slim/pkg/util/fsutil"
+	"github.com/slimtoolkit/slim/pkg/app"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/ipc"
+	"github.com/slimtoolkit/slim/pkg/app/master/inspectors/sensor"
+	"github.com/slimtoolkit/slim/pkg/app/sensor/standalone/control"
+	"github.com/slimtoolkit/slim/pkg/ipc/channel"
+	"github.com/slimtoolkit/slim/pkg/ipc/command"
+	"github.com/slimtoolkit/slim/pkg/ipc/event"
+	"github.com/slimtoolkit/slim/pkg/report"
+	"github.com/slimtoolkit/slim/pkg/util/fsutil"
 )
 
 const (
@@ -33,10 +35,13 @@ const (
 	// of the paths on the sensor side won't be unnoticed.
 	CommandsFileName  = "commands.json"
 	SensorLogFileName = "sensor.log"
+	MondelFileName    = "mondel.ndjson"
 	AppStdoutFileName = "app_stdout.log"
 	AppStderrFileName = "app_stderr.log"
 	EventsFileName    = "events.json"
 	runArchiveName    = "run.tar"
+
+	sensorExePath = "/opt/_slim/sensor"
 )
 
 var (
@@ -51,6 +56,12 @@ func WithSensorLogsToFile() sensorOpt {
 	}
 }
 
+func WithEnableMondel() sensorOpt {
+	return func(s *Sensor) {
+		s.enableMondel = true
+	}
+}
+
 func WithSensorArtifactsDir(dir string) sensorOpt {
 	return func(s *Sensor) {
 		s.artifactsDirPath = dir
@@ -75,6 +86,12 @@ func WithoutSensorCapabilities(caps ...string) sensorOpt {
 	}
 }
 
+func WithStopSignal(sig syscall.Signal) sensorOpt {
+	return func(s *Sensor) {
+		s.stopSignal = sig
+	}
+}
+
 type Sensor struct {
 	image          dockerapi.Image
 	contName       string
@@ -83,11 +100,13 @@ type Sensor struct {
 
 	// "Opts"
 	useLogFile       bool
+	enableMondel     bool
 	artifactsDirPath string
 	lifecycleHook    string
 	capAdd           []string
 	capDrop          []string
 	user             string
+	stopSignal       syscall.Signal
 
 	// "Nullable"
 	contID      string
@@ -99,6 +118,8 @@ type Sensor struct {
 	creport    *report.ContainerReport
 	rawCReport string
 	rawEvents  string
+	mondel     []report.MonitorDataEvent
+	rawMondel  string
 }
 
 func NewSensor(
@@ -169,10 +190,10 @@ func (s *Sensor) StartControlled(ctx context.Context) error {
 			[]string{
 				"--name", s.contName,
 				"--user", s.user,
-				"--volume", s.sensorExePath + ":/opt/_slim/sensor",
+				"--volume", s.sensorExePath + ":" + sensorExePath,
 				"--publish", fmt.Sprintf("%d", channel.CmdPort),
 				"--publish", fmt.Sprintf("%d", channel.EvtPort),
-				"--entrypoint", "/opt/_slim/sensor",
+				"--entrypoint", sensorExePath,
 			},
 		),
 		s.image.ID,
@@ -247,7 +268,9 @@ func (s *Sensor) StartStandalone(
 	s.rawCommands = string(rawCommands)
 
 	stopSignal := "SIGTERM"
-	if len(s.image.Config.StopSignal) > 0 {
+	if s.stopSignal != 0 {
+		stopSignal = unix.SignalName(s.stopSignal)
+	} else if len(s.image.Config.StopSignal) > 0 {
 		stopSignal = s.image.Config.StopSignal
 	}
 
@@ -264,9 +287,9 @@ func (s *Sensor) StartStandalone(
 			[]string{
 				"--name", s.contName,
 				"--user", s.user,
-				"--volume", s.sensorExePath + ":/opt/_slim/sensor",
+				"--volume", s.sensorExePath + ":" + sensorExePath,
 				"--volume", commandsFilePath + ":/opt/_slim/commands.json",
-				"--entrypoint", "/opt/_slim/sensor",
+				"--entrypoint", sensorExePath,
 			},
 		),
 		s.image.ID,
@@ -311,6 +334,16 @@ func (s *Sensor) StartStandaloneOrFail(
 }
 
 func (s *Sensor) SendCommand(ctx context.Context, cmd command.Message) error {
+	msg, err := command.Encode(cmd)
+	if err != nil {
+		return fmt.Errorf("cannot encode command %q: %w", cmd, err)
+	}
+	log.Debugf("Sending command to the test sensor: %s", string(msg))
+
+	if len(s.contID) == 0 {
+		return errNotStarted
+	}
+
 	if s.client == nil {
 		return errors.New("IPC client isn't initialized - is sensor running?")
 	}
@@ -352,6 +385,59 @@ func (s *Sensor) SendStopCommandOrFail(t *testing.T, ctx context.Context) {
 	}
 }
 
+func (s *Sensor) ExecuteControlCommand(ctx context.Context, cmd control.Command) error {
+	if len(s.contID) == 0 {
+		return errNotStarted
+	}
+
+	if s.client != nil {
+		return fmt.Errorf("cannot execute control command - sensor is not in the standalone mode")
+	}
+
+	if out, err := containerExec(
+		ctx,
+		s.contID,
+		sensorExePath,
+		"control",
+		string(cmd),
+	); err != nil {
+		return fmt.Errorf("cannot execute control command: %w\n%s", err, string(out))
+	}
+
+	return nil
+}
+
+func (s *Sensor) ExecuteControlCommandOrFail(t *testing.T, ctx context.Context, cmd control.Command) {
+	if err := s.ExecuteControlCommand(ctx, cmd); err != nil {
+		t.Fatalf("Failed executing control command %s: %v", cmd, err)
+	}
+}
+
+func (s *Sensor) WaitForEvent(ctx context.Context, evt event.Type) error {
+	if len(s.contID) == 0 {
+		return errNotStarted
+	}
+
+	if out, err := containerExec(
+		ctx,
+		s.contID,
+		sensorExePath,
+		"control",
+		string(control.WaitForEventCommand),
+		string(evt),
+	); err != nil {
+		return fmt.Errorf("cannot wait for sensor event %s: %w\n%s", evt, err, string(out))
+	}
+
+	return nil
+}
+
+func (s *Sensor) WaitForEventOrFail(t *testing.T, ctx context.Context, evt event.Type) {
+	if err := s.WaitForEvent(ctx, evt); err != nil {
+		t.Fatalf("Failed waiting for sensor event %s: %v", evt, err)
+	}
+}
+
 func (s *Sensor) Shutdown(ctx context.Context) error {
 	if err := s.SendCommand(ctx, &command.ShutdownSensor{}); err != nil {
 		return err
@@ -432,6 +518,24 @@ func (s *Sensor) DownloadArtifacts(ctx context.Context) error {
 		s.creport = &creport
 	}
 
+	if s.enableMondel {
+		mondelFilePath := filepath.Join(s.localArtifactsDirPath(), MondelFileName)
+		rawMondel, err := os.ReadFile(mondelFilePath)
+		if err != nil {
+			return fmt.Errorf("cannot read %s file: %w", MondelFileName, err)
+		}
+
+		s.rawMondel = string(rawMondel)
+
+		for _, line := range strings.Split(strings.TrimSpace(s.rawMondel), "\n") {
+			var evt report.MonitorDataEvent
+			if err := json.Unmarshal([]byte(line), &evt); err != nil {
+				return fmt.Errorf("cannot decode test sensor's mondel line %#q: %w", line, err)
+			}
+			s.mondel = append(s.mondel, evt)
+		}
+	}
+
 	if s.client == nil {
 		rawEvents, err := os.ReadFile(filepath.Join(s.localArtifactsDirPath(), EventsFileName))
 		if err != nil {
@@ -439,6 +543,7 @@ func (s *Sensor) DownloadArtifacts(ctx context.Context) error {
 		}
 		s.rawEvents = string(rawEvents)
 	}
+
 	return nil
 }
 
@@ -507,17 +612,18 @@ func (s *Sensor) PrintState(ctx context.Context) {
 		WithField("creport downloaded", s.creport != nil).
 		Info("Printing out test sensor state")
 
-	if s.creport != nil {
+	if len(s.rawCReport) > 0 {
 		fmt.Fprintln(os.Stderr, "-=== Container report ===-")
-		encoder := json.NewEncoder(os.Stderr)
-		encoder.SetEscapeHTML(false)
-		encoder.SetIndent("", "  ")
-		if err := encoder.Encode(s.creport); err != nil {
-			log.WithError(err).Error("Cannot print out container report")
-		}
+		fmt.Fprintln(os.Stderr, s.rawCReport)
 		fmt.Fprintln(os.Stderr, "-=== eof: Container report ===-")
 	}
 
+	if len(s.rawMondel) > 0 {
+		fmt.Fprintln(os.Stderr, "-=== Container MonDEL ===-")
+		fmt.Fprintln(os.Stderr, s.rawMondel)
+		fmt.Fprintln(os.Stderr, "-=== eof: Container MonDEL ===-")
+	}
+
 	if len(s.rawEvents) > 0 {
 		fmt.Fprintln(os.Stderr, "-=== events.json ===-")
 		fmt.Fprintln(os.Stderr, s.rawEvents)
@@ -677,6 +783,57 @@ func (s *Sensor) AssertReportNotIncludesFiles(t *testing.T, filepath ...string)
 	}
 }
 
+func (s *Sensor) AssertMondelIncludesFiles(t *testing.T, filepath ...string) {
+	if s.mondel == nil {
+		t.Fatal("No sensor mondel file found")
+	}
+
+	index := mondelEventByFilePath(s.mondel)
+	for _, f := range filepath {
+		if _, found := index[f]; !found {
+			t.Errorf("Expected file %q not found in the mondel file", f)
+		}
+	}
+}
+
+func (s *Sensor) AssertMondelNotIncludesFiles(t *testing.T, filepath ...string) {
+	if s.mondel == nil {
+		t.Fatal("No sensor mondel file found")
+	}
+
+	index := mondelEventByFilePath(s.mondel)
+	for _, f := range filepath {
+		if _, found := index[f]; found {
+			t.Errorf("Unexpected file %q found in the mondel file", f)
+		}
+	}
+}
+
+func (s *Sensor) AssertReportAndMondelFileListsMatch(t *testing.T) {
+	if s.creport == nil {
+		t.Fatal("No sensor report found")
+	}
+
+	if len(s.mondel) == 0 {
+		t.Fatal("No sensor mondel file found")
+	}
+
+	uniqMondelFiles := mondelEventByFilePath(s.mondel)
+	uniqCReportFiles := artifactsByFilePath(s.creport.Image.Files)
+
+	for f := range uniqMondelFiles {
+		if _, found := uniqCReportFiles[f]; !found {
+			t.Errorf("File %q found in mondel but not in container report", f)
+		}
+	}
+
+	for f := range uniqCReportFiles {
+		if _, found := uniqMondelFiles[f]; !found {
+			t.Errorf("File %q found in container report but not in mondel", f)
+		}
+	}
+}
+
 func (s *Sensor) AssertArtifactsArchiveContains(
 	t *testing.T,
 	ctx context.Context,
@@ -734,6 +891,9 @@ func (s *Sensor) commonStartFlags() []string {
 	if s.useLogFile {
 		flags = append(flags, "-o", filepath.Join(s.remoteArtifactsDirPath(), SensorLogFileName))
 	}
+	if s.enableMondel {
+		flags = append(flags, "-n")
+	}
 	if len(s.lifecycleHook) > 0 {
 		flags = append(flags, "-a", s.lifecycleHook)
 	}
@@ -852,6 +1012,14 @@ func artifactsByFilePath(files []*report.ArtifactProps) map[string]*report.Artif
 	return dict
 }
 
+func mondelEventByFilePath(events []report.MonitorDataEvent) map[string]report.MonitorDataEvent {
+	dict := make(map[string]report.MonitorDataEvent)
+	for _, evt := range events {
+		dict[evt.Artifact] = evt
+	}
+	return dict
+}
+
 func hostPort(cont dockerapi.Container, contPort int) (string, bool) {
 	if cont.NetworkSettings != nil {
 		for port, bindings := range cont.NetworkSettings.Ports {
