@@ -3,9 +3,9 @@ package execution
 import (
 	log "github.com/sirupsen/logrus"
 
-	"github.com/docker-slim/docker-slim/pkg/app/sensor/execution"
-	"github.com/docker-slim/docker-slim/pkg/ipc/command"
-	"github.com/docker-slim/docker-slim/pkg/ipc/event"
+	"github.com/slimtoolkit/slim/pkg/app/sensor/execution"
+	"github.com/slimtoolkit/slim/pkg/ipc/command"
+	"github.com/slimtoolkit/slim/pkg/ipc/event"
 )
 
 type executionStub struct {
@@ -20,6 +20,10 @@ func NewExecution() *executionStub {
 	}
 }
 
+func (e *executionStub) State() string {
+	return ""
+}
+
 func (e *executionStub) Commands() <-chan command.Message {
 	return e.commands
 }
@@ -51,6 +55,10 @@ func (e *executionStub) HookMonitorPreStart() {
 	// noop
 }
 
+func (e *executionStub) HookTargetAppRunning() {
+	// noop
+}
+
 func (e *executionStub) HookMonitorPostShutdown() {
 	// noop
 }
