@@ -15,8 +15,8 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"github.com/docker-slim/docker-slim/pkg/pdiscover"
-	"github.com/docker-slim/docker-slim/pkg/util/errutil"
+	"github.com/slimtoolkit/slim/pkg/pdiscover"
+	"github.com/slimtoolkit/slim/pkg/util/errutil"
 
 	"github.com/bmatcuk/doublestar"
 	log "github.com/sirupsen/logrus"
@@ -274,6 +274,32 @@ func IsTarFile(target string) bool {
 	return true
 }
 
+func HasReadAccess(dst string) (bool, error) {
+	err := unix.Access(dst, unix.R_OK)
+	if err == nil {
+		return true, nil
+	}
+
+	if err == unix.EACCES {
+		return false, nil
+	}
+
+	return false, err
+}
+
+func HasWriteAccess(dst string) (bool, error) {
+	err := unix.Access(dst, unix.W_OK)
+	if err == nil {
+		return true, nil
+	}
+
+	if err == unix.EACCES {
+		return false, nil
+	}
+
+	return false, err
+}
+
 // SetAccess updates the access permissions on the destination
 func SetAccess(dst string, access *AccessInfo) error {
 	if dst == "" || access == nil {
@@ -1018,9 +1044,13 @@ func CopyDirOnly(clone bool, src, dst string) error {
 
 // CopyDir copies a directory
 func CopyDir(clone bool,
-	src, dst string,
-	copyRelPath, skipErrors bool,
-	excludePatterns []string, ignoreDirNames, ignoreFileNames map[string]struct{}) (error, []error) {
+	src string,
+	dst string,
+	copyRelPath bool,
+	skipErrors bool,
+	excludePatterns []string,
+	ignoreDirNames map[string]struct{},
+	ignoreFileNames map[string]struct{}) (error, []error) {
 	log.Debugf("CopyDir(%v,%v,%v,%v,%#v,...)", src, dst, copyRelPath, skipErrors, excludePatterns)
 
 	if src == "" {
